/************************************************************************/
/* TaxSolve_CA_540_2012.c - California state 540 tax form.		*/
/* Copyright (C) 2012 - Aston Roberts					*/
/* 									*/
/* Compile:   gcc taxsolve_CA_540_2012.c -o taxsolve_CA_540_2012	*/
/* Run:	      ./taxsolve_CA_540_2012  CA_540_2012.txt 			*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-7-2013	aston_roberts@yahoo.com			*/
/************************************************************************/

float thisversion=10.0;

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "taxsolve_routines.c"

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       4
#define WIDOW		        5

int status=0;	/* Value for filing status. */


double TaxRateFormula( double income, int status )
{
 double tax;

 if ((status==SINGLE) || (status==MARRIED_FILLING_SEPARAT))
  {
   if (income <  7455.0)  tax = 	   0.01   *  income; 		  else 
   if (income < 17676.0)  tax =    74.55 + 0.02   * (income -   7455.0);  else 
   if (income < 27897.0)  tax =   278.97 + 0.04   * (income -  17676.0);  else 
   if (income < 38726.0)  tax =   687.81 + 0.06   * (income -  27897.0);  else 
   if (income < 48943.0)  tax =  1337.55 + 0.08   * (income -  38736.0);  else
   if (income < 250000.0) tax =  2154.83 + 0.093  * (income -  48943.0);  else 
   if (income < 300000.0) tax = 20853.22 + 0.103  * (income - 250000.0);  else 
   if (income < 500000.0) tax = 26003.22 + 0.113  * (income - 300000.0);
   else 	 	  tax = 48603.33 + 0.123  * (income - 500000.0);
  }
 else
 if ((status==MARRIED_FILLING_JOINTLY) || (status==WIDOW))
  {
   if (income <   14910.0) tax = 	    0.01   *  income;		    else 
   if (income <   35352.0) tax =   149.10 + 0.02   * (income -   14910.0);  else 
   if (income <   55794.0) tax =   557.94 + 0.04   * (income -   35352.0);  else 
   if (income <   77452.0) tax =  1375.62 + 0.06   * (income -   55794.0);  else 
   if (income <   97884.0) tax =  2675.10 + 0.08   * (income -   77452.0);  else
   if (income <  500000.0) tax =  4309.66 + 0.093  * (income -   97884.0);  else 
   if (income <  600000.0) tax = 41706.45 + 0.103  * (income -  500000.0);  else 
   if (income < 1000000.0) tax = 52006.45 + 0.113  * (income -  600000.0);
   else 		   tax = 97206.45 + 0.123  * (income - 1000000.0);
  }
 else
  {
   if (income <  14920.0) tax =		   0.01   *  income;		  else
   if (income <  35351.0) tax =   149.20 + 0.02   * (income -  14920.0);  else 
   if (income <  45571.0) tax =   557.82 + 0.04   * (income -  35351.0);  else 
   if (income <  56400.0) tax =   966.62 + 0.06   * (income -  45571.0);  else 
   if (income <  66618.0) tax =  1616.36 + 0.08   * (income -  56400.0);  else
   if (income < 340000.0) tax =  2433.80 + 0.093  * (income -  66618.0);  else
   if (income < 408000.0) tax = 27858.33 + 0.103  * (income - 340000.0);  else
   if (income < 680000.0) tax = 34862.33 + 0.113  * (income - 408000.0);  
   else 		  tax = 65598.33 + 0.123  * (income - 680000.0);
  }
 return (int)(tax+0.5);
}


double TaxRateFunction( double income, int status )     /* Emulates table lookup or function appropriately. */
{
 double x, tx;
 int k;

 if (income < 100000.0)   /* Quantize to match tax-table exactly. */
  {
   if (income < 99951.0) 
    {
     k = (income + 49) / 100;
     x = 100 * (double)k;
    }
   else x = 99975.0;
   tx = (int)(TaxRateFormula( x, status ));
  }
 else
  tx = TaxRateFormula( income, status );
 return tx;
}

/*----------------------------------------------------------------------------*/


struct FedReturnData
 {
  double fedline[MAX_LINES], schedA[MAX_LINES];
  int Exception, Itemized;
 } PrelimFedReturn;


void convert_slashes( char *fname )
{ /* Convert slashes in file name based on machine type. */
  char *ptr;
 #ifdef __MINGW32__
  char slash_sreach='/', slash_replace='\\';
 #else
  char slash_sreach='\\', slash_replace='/';
 #endif
  ptr = strchr( fname, slash_sreach );
  while (ptr)
   {
    ptr[0] = slash_replace;
    ptr = strchr( fname, slash_sreach );
   }
}


int ImportFederalReturnData( char *fedlogfile, struct FedReturnData *fed_data )
{
 FILE *infile;
 char fline[2000], word[2000];
 int linenum;

 for (linenum=0; linenum<MAX_LINES; linenum++) 
  { 
   fed_data->fedline[linenum] = 0.0;
   fed_data->schedA[linenum] = 0.0;
  }
 convert_slashes( fedlogfile );
 infile = fopen(fedlogfile, "r");
 if (infile==0)
  {
   printf("Error: Could not open Federal return '%s'\n", fedlogfile);
   fprintf(outfile,"Error: Could not open Federal return '%s'\n", fedlogfile);
   exit( 1 ); 
  }
 fed_data->Itemized = 1; /* Set initial default values. */
 read_line(infile,fline);  linenum = 0;
 while (!feof(infile))
  {
   if (strstr(fline,"Use standard deduction.")!=0) fed_data->Itemized = 0;
   next_word(fline, word, " \t=");
   if ((strstr(word,"L")==word) && (strstr(fline," = ")!=0))
    {
     if (strcmp(word,"L9b") != 0)
      {
       if (sscanf(&word[1],"%d",&linenum)!=1)
	{
	 printf("Error: Reading Fed line number '%s%s'\n",word,fline);
	 fprintf(outfile,"Error: Reading Fed line number '%s%s'\n",word,fline);
	}
       next_word(fline, word, " \t=");
       if (sscanf(word,"%lf", &fed_data->fedline[linenum])!=1)
	{
	 printf("Error: Reading Fed line %d '%s%s'\n",linenum,word,fline);
	 fprintf(outfile,"Error: Reading Fed line %d '%s%s'\n",linenum,word,fline);
	}
       if (verbose) printf("FedLin[%d] = %2.2f\n", linenum, fed_data->fedline[linenum]);
      }
    }
   else
   if ((strstr(word,"A")==word) && (strstr(word,"AMT")!=word) && (strstr(fline," = ")!=0))
    {
     if (sscanf(&word[1],"%d",&linenum)!=1)
      {
	printf("Error: Reading Fed line number '%s%s'\n",word,fline);
	fprintf(outfile,"Error: Reading Fed line number '%s%s'\n",word,fline);
      }
     next_word(fline, word, " \t=");
     if (sscanf(word,"%lf", &fed_data->schedA[linenum])!=1) 
       {
	printf("Error: Reading Fed schedA %d '%s%s'\n",linenum,word,fline);
	fprintf(outfile, "Error: Reading Fed schedA %d '%s%s'\n",linenum,word,fline);
       }
     if (verbose) printf("FedLin[%d] = %2.2f\n", linenum, fed_data->schedA[linenum]);
    }
   else
   if (strcmp(word,"Status") == 0)
    {
     next_word(fline, word, " \t=");
     if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
     if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
     if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
     if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
     if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
     else 
      { 
       printf("Error: unrecognized status '%s'. Exiting.\n", word); 
       fprintf(outfile,"Error: unrecognized status '%s'. Exiting.\n", word);
       return 0; 
      }
    }
   read_line(infile,fline);
  }
 fclose(infile);
 return 1;
}



/*----------------------------------------------------------------------------*/
/* ---				Main					  --- */
/*----------------------------------------------------------------------------*/
int main( int argc, char *argv[] )
{
 int argk, j, k, iline7, iline8, iline9, iline10;
 double exlim=0, min2file=0.0, sched540[MAX_LINES], threshA=0, threshB=0;
 char word[4000], outfname[4000], prelim_1040_outfilename[5000];
 time_t now;

 /* Decode any command-line arguments. */
 argk = 1;  k=1;
 while (argk < argc)
 {
  if (strcmp(argv[argk],"-verbose")==0)  { verbose = 1; }
  else
  if (k==1)
   {
    infile = fopen(argv[argk],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[argk]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[argk]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,"_out.txt"); else strcpy(&(outfname[j]),"_out.txt");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else
   {printf("Unknown command-line parameter '%s'\n", argv[argk]); exit(1);}
  argk = argk + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (j=0; j<MAX_LINES; j++) 
  { L[j] = 0.0;  sched540[j] = 0.0; }

 /* Accept parameters from input file. */
 /* Expect  CA-540 lines, something like:
	Title:  CA 540 1999 Return
	L12	34900.0  {Wages}
 */

 printf("CA-540 2012 - v%3.2f\n", thisversion);

 /* Accept Form's "Title" line, and put out with date-stamp for your records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 get_parameter( infile, 's', word, "FileName" );      /* Preliminary Fed Return Output File-name. */
 get_word(infile, prelim_1040_outfilename );
 ImportFederalReturnData( prelim_1040_outfilename, &PrelimFedReturn );

 /* Filing Status. */
 /* get_parameter(infile, kind, x, emssg ) */
 // get_parameter( infile, 's', word, "Status" );	/* Single, Married/joint, Married/sep, Head house, Widow(er) */
 // get_parameter( infile, 'l', word, "Status?");
 // if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 // if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
 // if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 // if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
 // if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
 // else
 //  { 
 //   printf("Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house, Widow(er)\nExiting.\n", word); 
 //   fprintf(outfile,"Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house, Widow(er)\nExiting.\n", word); 
 //   exit(1); 
 //  }
 switch (status)
 {
  case SINGLE: 			fprintf(outfile,"Status = Single (%d)\n", status); break;
  case MARRIED_FILLING_JOINTLY: fprintf(outfile,"Status = Married/Joint (%d)\n", status); break;
  case MARRIED_FILLING_SEPARAT: fprintf(outfile,"Status = Married/Sep (%d)\n", status); break;
  case HEAD_OF_HOUSEHOLD: 	fprintf(outfile,"Status = Head_of_Household (%d)\n", status); break;
  case WIDOW: 		  	fprintf(outfile,"Status = Widow(er) (%d)\n", status); break;
 }
 fprintf(outfile,"\nStep-2 fill-in box %d\n", status );

 /* Exemptions. */
 get_parameter( infile, 's', word, "L6" );	/* Are you a dependent? (yes/No). */
 get_parameter( infile, 'b', &j, "L6"); 
 L[6] = j;
 if (L[6]==0) fprintf(outfile," L6 = no\n"); else fprintf(outfile," L6 = yes, (check box on line 6).\n");

 if ((status==SINGLE) || (status==MARRIED_FILLING_SEPARAT) || (status==HEAD_OF_HOUSEHOLD))
  iline7 = 1; else iline7 = 2;
 if (L[6]!=0.0) iline7 = 0; /* <-- Possible exceptions here. */
 L[7] = 104.0 * iline7;
 showline(7);

 get_parameter( infile, 's', word, "L8" );	/* Blind?, 1 if you or spouse, 2 if both. */
 get_parameter( infile, 'i', &iline8, "L8" );
 L[8] = iline8 * 104.0;
 showline(8);

 get_parameter( infile, 's', word, "L9" );	/* Senior?, 1 if you or spouse, 2 if both. */
 get_parameter( infile, 'i', &iline9, "L9" );
 L[9] = iline9 * 104.0;
 showline(9);

 get_parameter( infile, 's', word, "L10" );  /* Number of Dependents. */
 get_parameter( infile, 'i', &iline10, "L10"); 
 L[10] = iline10 * 321.0;
 showline(10);

 L[11] = L[7] + L[8] + L[9] + L[10];
 showline_wmsg(11,"Exemption amount");

 /* Taxable Income. */
 GetLineF( "L12", &L[12] );	/* State Wages (W2 box 16). */

 L[13] = PrelimFedReturn.fedline[37];	/* Fed Wages (Fed 1040 line 37). */

 /* -- Sched540 Part II -- */

 sched540[38] = PrelimFedReturn.schedA[29];
 sched540[39] = PrelimFedReturn.schedA[5] + PrelimFedReturn.schedA[8];
 sched540[40] = sched540[38] - sched540[39];
 GetLine( "Adj", &sched540[41] ); 
 sched540[42] = sched540[40] + sched540[41];
 switch (status)
  {
   case SINGLE:
   case MARRIED_FILLING_SEPARAT:  threshA = 169730.0;	threshB = 3841.0;  break;
   case HEAD_OF_HOUSEHOLD:        threshA = 254599.0;	threshB = 7682.0;  break;
   case MARRIED_FILLING_JOINTLY:
   case WIDOW:                    threshA = 339464.0;	threshB = 7682.0;  break;
  }
 if (L[13] > threshA)
  { /*Deductions Worksheet*/
    double ws[40];
    ws[1] = PrelimFedReturn.schedA[29];
    ws[2] = PrelimFedReturn.schedA[5] + PrelimFedReturn.schedA[8];	/* Plus SDI, assumed zero. */
    ws[3] = ws[1] - ws[2];	/* Total California Itemized Deductions. */
    for (j=1; j <= 3; j++)
     fprintf(outfile," ItemDeducWS[%d] = %6.2f\n", j, ws[j] );
    if (L[13] > threshA)
     {
	fprintf(outfile," ItemDeducWS[4] = Yes\n" );
	ws[5] = PrelimFedReturn.schedA[4] + PrelimFedReturn.schedA[14]
		+ PrelimFedReturn.schedA[20] + PrelimFedReturn.schedA[28];
	ws[6] = ws[3] - ws[5];
	if (ws[6] != 0.0)
	 {
	  ws[7] = 0.8 * ws[6];
	  ws[8] = L[13];
	  ws[9] = threshA;
	  ws[10] = ws[8] - ws[9];
	  if (ws[10] > 0.0)
	   {
	    ws[11] = 0.06 * ws[10];
	    ws[12] = smallerof( ws[7], ws[11] );
	    ws[13] = ws[3] - ws[12];
	   }
	  else
	   ws[13] = ws[3];
	 }
	else
	 ws[13] = ws[3];
	L[18] = largerof( ws[13], threshB );
     }
    else
     {
	fprintf(outfile," ItemDeducWS[4] = No\n");
	L[18] = largerof( ws[3], threshB );
     }
  } /*Deductions Worksheet*/
 else
  sched540[43] = sched540[42];
 sched540[44] = largerof( sched540[43], threshB );

 for (j=38; j <= 44; j++)	/* Display the worksheet calculations. */
  fprintf(outfile," SchedCA540_%d = %6.2f\n", j, sched540[j] );

 L[18] = sched540[44];



 /* -- End Sched540 Part II -- */

 GetLineF( "L14", &L[14] );	/* CA Adjustments, Schedule CA 540 line 37 column B. */

 L[15] = L[13] - L[14];
 if (L[15] < 0.0) fprintf(outfile,"L15 = (%f6.2)\n", -L[15] );
 else showline(15);

 GetLineF( "L16", &L[16] );	/* CA Adjustments, Schedule CA 540 line 37 column C. */

 L[17] = L[15] + L[16];		/* CA Adjusted Gross Income (AGI). */
 showline(17);

 switch (status)
  {
   case SINGLE: 
   case HEAD_OF_HOUSEHOLD: 	
   case WIDOW: 		  	
		if (iline9 == 0)
		   switch (iline10)		 /*under65*/
		    {
		     case 0:  if (status == WIDOW) min2file = 0.0;
			      else min2file = 12352.0;
			break;
		     case 1:  min2file = 23052.0;	break;
		     default: min2file = 31077.0;	break;
		    }
		else
		   switch (iline10)		 /*65+*/
		    {
		     case 0:  if (status == WIDOW) min2file = 0.0;
			      else min2file = 17552.0;
			break;
		     case 1:  min2file = 25577.0;	break;
		     default: min2file = 31997.0;	break;
		    }
	  break;
   case MARRIED_FILLING_JOINTLY: 
		if (iline9 == 0)
		   switch (iline10)		 /*Both under65*/
		    {
		     case 0:  min2file = 24705.0;	break;
		     case 1:  min2file = 35405.0;	break;
		     default: min2file = 43430.0;	break;
		    }
		else
		if (iline9 == 1)
		   switch (iline10)		 /*one 65+*/
		    {
		     case 0:  min2file = 29905.0;	break;
		     case 1:  min2file = 37930.0;	break;
		     default: min2file = 44350.0;	break;
		    }
		else
		   switch (iline10)		 /*both 65+*/
		    {
		     case 0:  min2file = 35105.0;	break;
		     case 1:  min2file = 43130.0;	break;
		     default: min2file = 49550.0;	break;
		    }
	  break;
  }
 if (L[17] <= min2file)
  fprintf(outfile,"You may not need to file CA Taxes, due to your California Adjusted Gross Income (%6.2f <= %6.2f).\n", L[17], min2file );

 showline(18);

 L[19] = NotLessThanZero( L[17] - L[18] );
 showline_wmsg(19,"Taxable Income");		/* Taxable income. */

 /* Tax. */
 if (L[19] < 100000.00)
  fprintf(outfile,"Fill in circle from: Tax Table.\n");
 else
  fprintf(outfile,"Fill in circle from: Tax Rate Schedule.\n");
 L[31] = TaxRateFunction( L[19], status );
 showline( 31 );

 switch (status)
  {
   case SINGLE:  case MARRIED_FILLING_SEPARAT:  exlim = 169730.0;  break;
   case MARRIED_FILLING_JOINTLY:  case WIDOW:   exlim = 339464.0;  break;
   case HEAD_OF_HOUSEHOLD:                      exlim = 254599.0;  break;
  }

 if (L[13] > exlim)
  { /*Line32-Exemption-credits-worksheet*/
    double ws_a, ws_b, ws_c, ws_d, ws_e, ws_f, ws_g, ws_h, ws_i, ws_j, ws_k, ws_l, ws_m, ws_n;
    printf(" Doing AGI Limitations worksheet.\n"); 
    ws_a = L[13];
    ws_b = exlim;
    ws_c = ws_a - ws_b;
    if (status != MARRIED_FILLING_SEPARAT)
	ws_d = Round(ws_c / 2500.0);  
    else 
	ws_d = Round(ws_c / 1250.0);
    ws_e = 6.0 * ws_d;  
    ws_f = iline7 + iline8 + iline9; 
    ws_g = ws_e * ws_f;
    ws_h = L[7] + L[8] + L[9];
    ws_i = NotLessThanZero( ws_h - ws_g );
    ws_j = iline10;
    ws_k = ws_e * ws_j;
    ws_l = L[10];
    ws_m = NotLessThanZero( ws_l - ws_k );
    ws_n = ws_i + ws_m;
    fprintf(outfile," AGI Worksheet:\n   a: %6.2f\n", ws_a);
    fprintf(outfile,"  b: %6.2f\n", ws_b);
    fprintf(outfile,"  c: %6.2f\n", ws_c);
    fprintf(outfile,"  d: %6.2f\n", ws_d);
    fprintf(outfile,"  e: %6.2f\n", ws_e);
    fprintf(outfile,"  f: %6.2f\n", ws_f);
    fprintf(outfile,"  g: %6.2f\n", ws_g);
    fprintf(outfile,"  h: %6.2f\n", ws_h);
    fprintf(outfile,"  i: %6.2f\n", ws_i);
    fprintf(outfile,"  j: %6.2f\n", ws_j);
    fprintf(outfile,"  k: %6.2f\n", ws_k);
    fprintf(outfile,"  l: %6.2f\n", ws_l);
    fprintf(outfile,"  m: %6.2f\n", ws_m);
    fprintf(outfile,"  n: %6.2f\n", ws_n);
    fprintf(outfile," Your exemptions may be limited. Used Exemptions-Credits-Worksheet for Line 21.\n");
    fprintf(outfile,"   WorkSheet[n]=%6.2f (vs. L11=%6.2f)\n",ws_n, L[11]);
    L[32] = ws_n;
  }
 else  L[32] = L[11];
 showline(32);

 L[33] = NotLessThanZero( L[31] - L[32] );
 showline(33);

 GetLineF( "L34", &L[34] );
 showline(34);		/* Taxes on distributions (sched G-1 or form FTB 5870A) */

 L[35] = L[33] + L[34];
 showline(35);

 L[36] = L[35];		/*(next-page)*/
 showline(36);

 /* Special Credits. */
 GetLineF( "L40", &L[40] );	/* Nonrefundable Child + Dependent Care Expenses Credit (pg 11). */
 fprintf(outfile," "); /* Indent next entry. */
 GetLineF( "L41", &L[41] );	/* Special credit 1 */
 GetLineF( "L42", &L[42] );	/* Special credit 2 */
 GetLineF( "L43", &L[43] );	/* Special credit 3 */
 GetLineF( "L44", &L[44] );	/* Special credit 4 */
 GetLineF( "L45", &L[45] );	/* Special credit 5+ */
 GetLineF( "L46", &L[46] );	/* Nonrefundable renter's credit */

 L[47] = L[40] + L[42] + L[43] + L[44] + L[45] + L[46];
 showline(47);			/* Total credits. */

 L[48] = NotLessThanZero( L[36] - L[47] );
 showline(48);

 /* Other taxes. */
 GetLineF( "L61", &L[61] );	/* Alternative minimum tax Sched P. */

 GetLineF( "L62", &L[62] );	/* Mental Health Services Tax. */

 GetLineF( "L63", &L[63] );	/* Other taxes and credit recapture. */

 L[64] = L[48] + L[61] + L[62] + L[63];
 showline_wmsg(64,"Total Tax");	/* Total tax. */

 /* Payments. */
 GetLineF( "L71", &L[71] ); 	/* CA income tax withheld. */
 
 GetLineF( "L72", &L[72] ); 	/* Estimated tax paid. */
 
 GetLineF( "L73", &L[73] ); 	/* Realestate withholding. */
 
 GetLineF( "L74", &L[74] ); 	/* Excess SDI. */
 
 L[75] = L[71] + L[72] + L[73] + L[74];
 showline_wmsg(75,"Total Payments");

 /* Refund / Tax-Due. */
 if (L[75] > L[64])
  {
   L[91] = L[75] - L[64];
   fprintf(outfile,"L91 = %6.2f  REBATE!!!\n", L[91] );
  }
 else
  {
   L[94] = L[64] - L[75];
   fprintf(outfile,"L94 = %6.2f  YOU OWE !!!\n", L[94] );
  }
 
 fprintf(outfile,"\nSelect any charity contributions and complete\n form accordingly.\n");

 fclose(infile);
 fclose(outfile);
 Display_File( outfname );
 return 0;
}
