/************************************************************************/
/* TaxSolve_NY_IT-201_2011.c - NY State Tax form IT-201 for 2011.	*/
/* Copyright (C) 2003-2011 - Aston Roberts, Skeet Monker		*/
/* 									*/
/* Compile:   gcc taxsolve_NY_IT201_2011.c -o taxsolve_NY_IT201_2011	*/
/* Run:	      ./taxsolve_NY_IT201_2011  NY_IT201_2011.txt 		*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 4-8-2003	aston_roberts@yahoo.com			*/
/* Modified for NY 2003 taxes - Skeet Monker				*/
/* Modified for NY 2004 taxes - A Roberts				*/
/* Modified for NY 2005-2011 taxes - Skeet Monker			*/
/************************************************************************/

float thisversion=10.00;

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "taxsolve_routines.c"

double A[10], S[10];

#define SINGLE                  1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       4
#define WIDOW                   5
#define Yes 1
#define No  0

int status=0;

char statusnames[10][20]={"0","Single","Married/Joint","Married/Sep","Head_of_House","Widow"};

struct FedReturnData
 {
  double fedline[MAX_LINES], schedA[MAX_LINES], schedD[MAX_LINES];
  int Exception, Itemized;
 } PrelimFedReturn;


void convert_slashes( char *fname )
{ /* Convert slashes in file name based on machine type. */
  char *ptr;
 #ifdef __MINGW32__
  char slash_sreach='/', slash_replace='\\';
 #else
  char slash_sreach='\\', slash_replace='/';
 #endif

  ptr = strchr( fname, slash_sreach );
  while (ptr)
   {
    ptr[0] = slash_replace;
    ptr = strchr( fname, slash_sreach );
   }
}


int ImportFederalReturnData( char *fedlogfile, struct FedReturnData *fed_data )
{
 FILE *infile;
 char fline[1000], word[1000];
 int linenum;

 for (linenum=0; linenum<MAX_LINES; linenum++) 
  { 
   fed_data->fedline[linenum] = 0.0;
   fed_data->schedA[linenum] = 0.0;
   fed_data->schedD[linenum] = 0.0;
  }
 convert_slashes( fedlogfile );
 infile = fopen(fedlogfile, "r");
 if (infile==0)
  {
   printf("Error: Could not open Federal return '%s'\n", fedlogfile);
   fprintf(outfile,"Error: Could not open Federal return '%s'\n", fedlogfile);
   return 0; 
  }
 fed_data->Itemized = 1; /* Set initial default values. */
 read_line(infile,fline);  linenum = 0;
 while (!feof(infile))
  {
   if (strstr(fline,"Use standard deduction.")!=0) fed_data->Itemized = 0;
   next_word(fline, word, " \t=");
   if ((strstr(word,"L")==word) && (strstr(fline," = ")!=0))
    {
     if (strcmp(word,"L9b") != 0)
      {
       if (sscanf(&word[1],"%d",&linenum)!=1)
	{
	 printf("Error: Reading Fed line number '%s%s'\n",word,fline);
	 fprintf(outfile,"Error: Reading Fed line number '%s%s'\n",word,fline);
	}
       next_word(fline, word, " \t=");
       if (sscanf(word,"%lf", &fed_data->fedline[linenum])!=1)
	{
	 printf("Error: Reading Fed line %d '%s%s'\n",linenum,word,fline);
	 fprintf(outfile,"Error: Reading Fed line %d '%s%s'\n",linenum,word,fline);
	}
       if (verbose) printf("FedLin[%d] = %2.2f\n", linenum, fed_data->fedline[linenum]);
      }
    }
   else
   if ((strstr(word,"A")==word) && (strstr(word,"AMT")!=word) && (strstr(fline," = ")!=0))
    {
     if (sscanf(&word[1],"%d",&linenum)!=1)
      {
	printf("Error: Reading Fed line number '%s%s'\n",word,fline);
	fprintf(outfile,"Error: Reading Fed line number '%s%s'\n",word,fline);
      }
     next_word(fline, word, " \t=");
     if (sscanf(word,"%lf", &fed_data->schedA[linenum])!=1) 
       {
	printf("Error: Reading Fed schedA %d '%s%s'\n",linenum,word,fline);
	fprintf(outfile, "Error: Reading Fed schedA %d '%s%s'\n",linenum,word,fline);
       }
     if (verbose) printf("FedLin[%d] = %2.2f\n", linenum, fed_data->schedA[linenum]);
    }
   else
   if ((strstr(word,"D")==word) && (strstr(fline," = ")!=0))
    {
     if (sscanf(&word[1],"%d",&linenum)!=1)
      {
	printf("Error: Reading Fed line number '%s%s'\n",word,fline);
	fprintf(outfile,"Error: Reading Fed line number '%s%s'\n",word,fline);
      }
     next_word(fline, word, " \t=");
     if (sscanf(word,"%lf", &fed_data->schedD[linenum])!=1) 
      {
       if (strncasecmp(word,"yes",1) == 0) fed_data->schedD[linenum] = 1;
       else
       if (strncasecmp(word,"no",1) == 0) fed_data->schedD[linenum] = 0;
       else
        {
         printf("Error: Reading fed schedD %d '%s%s'\n",linenum,word,fline);
	 fprintf(outfile,"Error: Reading Fed schedD %d '%s%s'\n",linenum,word,fline);
        }
      }
     if (verbose) printf("FedLin[%d] = %2.2f\n", linenum, fed_data->schedD[linenum]);
    }
   else
   if (strcmp(word,"Status") == 0)
    {
     next_word(fline, word, " \t=");
     if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
     if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
     if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
     if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
     if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
     else 
      { 
       printf("Error: unrecognized status '%s'. Exiting.\n", word); 
       fprintf(outfile,"Error: unrecognized status '%s'. Exiting.\n", word);
       return 0; 
      }
    }
   read_line(infile,fline);
  }
 fclose(infile);
 return 1;
}


double TaxRateFunction( double income, int status )
{
 double tax;

 switch (status)
  {
   case MARRIED_FILLING_JOINTLY: case WIDOW:
	if (income <= 16000.0) tax = 0.04 * income; else
	if (income <= 22000.0) tax = 640.0 + 0.045  * (income - 16000.0); else
	if (income <= 26000.0) tax = 910.0 + 0.0525 * (income - 22000.0); else
	if (income <= 40000.0) tax = 1120.0 + 0.059 * (income - 26000.0); else
	if (income <= 300000.0) tax = 1946.0 + 0.0685 * (income - 40000.0); else
	if (income <= 500000.0) tax = 19756.0 + 0.0785 * (income - 300000.0); else
	tax = 35456.0 + 0.0897 * (income - 500000.0);
      break;
   case SINGLE: case MARRIED_FILLING_SEPARAT:
	if (income <=  8000.0) tax =  	     0.04   * income; else
	if (income <= 11000.0) tax =  320.0 + 0.045  * (income - 8000.0); else
	if (income <= 13000.0) tax =  455.0 + 0.0525 * (income - 11000.0); else
	if (income <= 20000.0) tax =  560.0 + 0.059  * (income - 13000.0); else
	if (income <= 200000.0) tax =  973.0 + 0.0685 * (income - 20000.0); else
	if (income <= 500000.0) tax =  13303.0 + 0.0785 * (income - 200000.0); else
	tax =  36853.0 + 0.0897 * (income - 500000.0);
      break;
   case HEAD_OF_HOUSEHOLD:
	if (income <= 11000.0) tax = 0.04 * income; else
	if (income <= 15000.0) tax = 440.0 + 0.045  * (income - 11000.0); else
	if (income <= 17000.0) tax = 620.0 + 0.0525 * (income - 15000.0); else
	if (income <= 30000.0) tax = 725.0 + 0.059  * (income - 17000.0); else
	if (income <= 250000.0) tax = 1492.0 + 0.0685 * (income - 30000.0); else
	if (income <= 500000.0) tax = 16562.0 + 0.0785 * (income - 250000.0); else
	tax = 36187.0 + 0.0897 * (income - 500000.0);
      break;
   default: printf("Error: Unhandled status\n"); exit(0); break;
  }
 return tax;
}


double TaxRateLookup( double income, int status )
{
 double tax, dx;
 int m;

 if (income < 25.0)  dx = 12.5;  else
 if (income < 50.0)  dx = 25.0;  else  dx = 50.0;

 /* Round and truncate results from tax-function to approximate table lookup. */
 m = income / dx;             /* Round income to nearest $50. */
 income = (double)m * dx + 0.5 * dx;      /* Place into center of a $50 bracket. */
 tax = TaxRateFunction( income, status );

 return (int)(tax + 0.5);
}


double NYcityTaxRateFunction( double income, int status )
{
 double tax, dx;
 int m;

 if (income<25.0) dx = 12.5; else
 if (income<50.0) dx = 25.0; else dx = 50.0;

 m = income / dx;             /* Round income to nearest $50. */
 if (income<65000.0)
  income = m * dx + 0.5 * dx;      /* Place into center of a $50 bracket. */

 if ((status==MARRIED_FILLING_JOINTLY) || (status==WIDOW))
  {
   if (income < 21600.0)  tax = income * 0.02907; else
   if (income < 45000.0)  tax = (income - 21600.00) * 0.03534 + 628.00; else
   if (income < 90000.0)  tax = (income - 45000.00) * 0.03591 + 1455.0; else
   if (income < 500000.0) tax = (income - 90000.00) * 0.03648 + 3071.0; else
   tax = (income - 500000.0) * 0.03876 + 18028.00;
  }
 else
 if ((status==SINGLE) || (status==MARRIED_FILLING_SEPARAT))
  {
   if (income < 12000.0) tax = income * 0.02907; else
   if (income < 25000.0) tax = (income - 12000.00) * 0.03534 + 349.0; else
   if (income < 50000.0) tax = (income - 25000.00) * 0.03591 + 808.0; else
   if (income < 500000.0) tax = (income - 50000.00) * 0.03648 + 1706.0; else
   tax = (income - 500000.0) * 0.03876 + 18122.00;
  }
 else
 if (status==HEAD_OF_HOUSEHOLD) 
  {
   if (income < 14000.00) tax = income * 0.02907; else
   if (income < 30000.00) tax = (income - 14000.00) * 0.03534 + 419.0; else
   if (income < 60000.00) tax = (income - 30000.00) * 0.03591 + 970.0; else
   if (income < 500000.0) tax = (income - 60000.00) * 0.03648 + 2047.0;
   tax = (income - 500000.0) * 0.03876 + 18098.00;
 }
 else {printf("Status not covered.\n");  exit(1);}

 if (income<65000.0) tax = (int)(tax + 0.5);   /* Round result to whole dollar. */
 return tax;
}



double tax_computation_worksheet( int status )
{ /* Worksheets pages 60-61. */
 double ws[100], agi_threshold, adder;

 switch (status)
  {
     case MARRIED_FILLING_JOINTLY:  case WIDOW:
	agi_threshold = 300000.0; 
	adder = 794.0;
	break;
     case SINGLE:  case MARRIED_FILLING_SEPARAT:
	agi_threshold = 200000.0;
	adder = 397.0;
	break;
     case HEAD_OF_HOUSEHOLD:
	agi_threshold = 250000.0;
	adder = 563.0;
	break;
     default: printf("Case not handled.\n");  fprintf(outfile,"Case not handled.\n"); exit(1);
  }

 if (L[33] <= 150000.0)
  { /*Tax Computation Worksheet 1 (pg 60) */
   ws[1] = L[33];
   ws[2] = L[38];
   ws[3] = 0.0685 * ws[2];
   ws[4] = TaxRateFunction( ws[2], status );
   ws[5] = ws[3] - ws[4];
   ws[6] = smallerof( ws[1] - 100000.0, 50000.0 );
   /* Divide by 50k and round to forth decimal place. */
   ws[7] = 0.0001 * (double)Round( 10000.0 * (ws[6] / 50000.0) );
   ws[8] = ws[5] * ws[7];
   ws[9] = ws[4] + ws[8];
   return ws[9];	//L[39] = ws[9];
  }
 else 
 if (L[33] <= 500000.0)
  {
   if (L[38] <= agi_threshold)
    {/*Tax Computation Worksheet 2 (pg 60) */
      return 0.0685 * L[38];	// L[39] = 0.0685 * L[38];
    }
   else
    {/*Tax Computation Worksheet 3 (pg 50) */
      ws[1] = L[33];
      ws[2] = L[38];
      ws[3] = 0.0785 * ws[2];
      ws[4] = TaxRateFunction( ws[2], status );
      ws[5] = ws[3] - ws[4];
      ws[6] = adder;
      ws[7] = ws[5] - ws[6];
      if (ws[1] <= 300000.0)
       {
	ws[8] = 0.0;
	ws[9] = 0.0;
	ws[10] = 0.0;
       }
      else
       {
	ws[8] = smallerof( ws[1] - 300000.0, 50000.0 );
	/* Divide by 50k and round to forth decimal place. */
	ws[9] = 0.0001 * (double)Round( 10000.0 * (ws[8] / 50000.0) );
	ws[10] = ws[7] * ws[9];
       }
      ws[11] = ws[6];
      ws[12] = ws[4] + ws[10] + ws[11];
      return ws[12];	// L[39] = ws[12];
    }
  }
 else 
 if (L[33] <= 550000.0)
  {
   if (L[38] <= agi_threshold)
    {/*Tax Computation Worksheet 4 (pg 61) */
      ws[1] = L[33];
      ws[2] = L[38];
      ws[3] = 0.0897 * ws[2];
      ws[4] = TaxRateFunction( ws[2], status );
      ws[5] = ws[3] - ws[4];
      ws[6] = adder;
      ws[7] = ws[5] - ws[6];
      if (ws[1] <= 500000.0)
       {
	ws[8] = 0.0;
	ws[9] = 0.0;
	ws[10] = 0.0;
       }
      else
       {
	ws[8] = smallerof( ws[1] - 500000.0, 50000.0 );
	/* Divide by 50k and round to forth decimal place. */
	ws[9] = 0.0001 * (double)Round( 10000.0 * (ws[8] / 50000.0) );
	ws[10] = ws[7] * ws[9];
       }
      ws[11] = ws[6];
      ws[12] = smallerof( ws[4] + ws[10] + ws[11], ws[3] );
      return ws[12];	// L[39] = ws[12];
    }
   else
    {/*Tax Computation Worksheet 5 (pg 62) */
      ws[1] = L[33];
      ws[2] = L[38];
      ws[3] = 0.0897 * ws[2];
      ws[4] = TaxRateFunction( ws[2], status );
      ws[5] = ws[3] - ws[4];
      switch (status)
       {
        case MARRIED_FILLING_JOINTLY:  case WIDOW:  ws[6] = 3794.0; break;
        case SINGLE:  case MARRIED_FILLING_SEPARAT: ws[6] = 2397.0; break;
        case HEAD_OF_HOUSEHOLD:                     ws[6] = 3063.0; break;
        default: printf("Case not handled.\n"); fprintf(outfile,"Case not handled.\n"); exit(1);
       }
      ws[7] = ws[5] - ws[6];
      if (ws[1] <= 500000.0)
       {
	ws[8] = 0.0;
	ws[9] = 0.0;
	ws[10] = 0.0;
       }
      else
       {
	ws[8] = smallerof( ws[1] - 500000.0, 50000.0 );
	/* Divide by 50k and round to forth decimal place. */
	ws[9] = 0.0001 * (double)Round( 10000.0 * (ws[8] / 50000.0) );
	ws[10] = ws[7] * ws[9];
       }
      ws[11] = ws[6];
      ws[12] = ws[4] + ws[10] + ws[11];
      return ws[12];	// L[39] = ws[12];
    }
  }
 else
  { /*Tax Computation Worksheet 6 (pg 61) */
    ws[1] = L[38];
    ws[2] = 0.0897 * ws[1];
    return ws[2];	// L[39] = ws[2];
  }
}


/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int j, k, argk;
 char word[1000], outfname[1000];
 time_t now;
 int Dependent, Exemptions, nyc_resident;
 double itemized_ded, std_ded=0.0, LTC=0, AddAdj=0.0, CollegeDed=0.0;
 double dedws_a=0.0, dedws_b=0.0, dedws_c=0.0, dedws_d=0.0, dedws_e=0.0, dedws_f=0.0,
	dedws_g=0.0, dedws_h=0.0, dedws_i=0.0, dedws_j=0.0, dedws_k=0.0, dedws_l=0.0,
	dedws_m=0.0, dedws_n=0.0, dedws_o=0.0, dedws_p=0.0;
 char prelim_1040_outfilename[5000];

 /* Intercept any command-line arguments. */
 printf("NY-IT201 - 2011 - v%3.1f\n", thisversion);
 argk = 1;  k=1;
 while (argk < argc)
 {
  if (strcmp(argv[argk],"-verbose")==0)  verbose = 1;
  else
  if (k==1)
   {
    infile = fopen(argv[argk],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[argk]);  exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[argk]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,"_out.txt"); else strcpy(&(outfname[j]),"_out.txt");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname);  exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else {printf("Unknown command-line parameter '%s'\n", argv[argk]);  exit(1);}
  argk = argk + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (j=0; j<MAX_LINES; j++)  L[j] = 0.0;

 /* Accept parameters from input file. */
 /* Expect  NY IT200 form lines, something like:
	Title:  NY IT-200 1999 Return
	L1	;	{Wages}
	L2	;	{Interest}
	L3	;	{Dividends}
	...
*/


 /* Accept Form's "Title" line, and put out with date-stamp for records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 get_parameter( infile, 's', word, "FileName" );      /* Preliminary Return Output File-name. */
 get_word(infile, prelim_1040_outfilename );
 if (ImportFederalReturnData( prelim_1040_outfilename, &PrelimFedReturn ) == 0)
  {
   fclose(infile);
   fclose(outfile);
   Display_File( outfname );
   exit(1);
  }

 /* get_parameter(infile, kind, x, emssg ) */
 // get_parameter( infile, 's', word, "Status" );
 // get_parameter( infile, 'l', word, "Status?");
 // if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 // if (strncasecmp(word,"Married/Joint",13)==0) status = MARRIED_FILLING_JOINTLY; else
 // if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 // if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD; else
 // if (strncasecmp(word,"Widow",4)==0) status = WIDOW;
 // else { printf("Error: unrecognized status '%s'. Exiting.\n", word); exit(1); }
 fprintf(outfile,"Status = %s (%d)\n", statusnames[status], status);

 get_parameter( infile, 's', word, "Dependent" );
 get_parameter( infile, 'b', &Dependent, "Dependent?"); 
 if (Dependent==1) fprintf(outfile," Check box C = Yes\n");  else fprintf(outfile," Check box C = No\n");

 // GetLineF( "L1", &L[1] );	/* Wages. */
 L[1] = PrelimFedReturn.fedline[7];
 showline(1);

 // GetLineF( "L2", &L[2] );	/* Taxable Interest. */
 L[2] = PrelimFedReturn.fedline[8];
 showline(2);

 // GetLineF( "L3", &L[3] );	/* Ordinary Dividends. */
 L[3] = PrelimFedReturn.fedline[9];
 showline(3);

 // GetLineF( "L4", &L[4] );	/* Taxable refunds, credits, offsets */
 L[4] = PrelimFedReturn.fedline[10];
 showline(4);

 // GetLineF( "L5", &L[5] );	/* Alimony received */
 L[5] = PrelimFedReturn.fedline[11];
 showline(5);

 // GetLineF( "L6", &L[6] );	/* Business income/loss (fed sched C) */
 L[6] = PrelimFedReturn.fedline[12];
 showline(6);

 // GetLineF( "L7", &L[7] );	/* Capital gains/losses (fed sched D) */
 L[7] = PrelimFedReturn.fedline[13];
 showline(7);

 // GetLineF( "L8", &L[8] );	/* Other gains/losses (fed form 4794) */
 L[8] = PrelimFedReturn.fedline[14];
 showline(8);

 // GetLineF( "L9", &L[9] );	/* Taxable IRA distributions */
 L[9] = PrelimFedReturn.fedline[15];
 showline(9);

 // GetLineF( "L10", &L[10] );	/* Taxable pension/annuity amounts  */
 L[10] = PrelimFedReturn.fedline[16];
 showline(10);

 // GetLineF( "L11", &L[11] );	/* Rental, royalties, partnership, S corp, (fed sched E) */
 L[11] = PrelimFedReturn.fedline[17];
 showline(11);

 // GetLineF( "L12", &L[12] );	/* Farm income (fed sched F) */
 L[12] = PrelimFedReturn.fedline[18];
 showline(12);

 // GetLineF( "L13", &L[13] );	/* Unemployment compensation */
 L[13] = PrelimFedReturn.fedline[19];
 showline(13);

 // GetLineF( "L14", &L[14] );	/* Taxable Social Sec. benefits */
 L[14] = PrelimFedReturn.fedline[20];
 showline(14);
 L[27] = L[14];

 // GetLineF( "L15", &L[15] );	/* Other income (pg. 15) */
 L[15] = PrelimFedReturn.fedline[21];
 showline(15);

 for (j=1; j<=15; j++) L[16] = L[16] + L[j];
 showline(16);
 if (absolutev( L[16] - PrelimFedReturn.fedline[22]) > 0.1)
  {
   printf(" Warning: L[16] = %6.2f, while Fed-line[22] = %6.2f\n", L[16], PrelimFedReturn.fedline[22] );
   fprintf(outfile," Warning: L[16] = %6.2f, while Fed-line[22] = %6.2f\n", L[16], PrelimFedReturn.fedline[22] );
  }

 // GetLineF( "L17", &L[17] );	/* Total federal adjustments to income (pg 15) */
 L[17] = PrelimFedReturn.fedline[36];
 showline(17);

 L[18] = L[16] - L[17];
 showline_wmsg(18,"Federal adjusted gross income");
 if (absolutev(L[18] - PrelimFedReturn.fedline[38]) > 0.1)
  {
   printf(" Warning: L[18] = %6.2f, while Fed-line[38] = %6.2f\n", L[18], PrelimFedReturn.fedline[38] );
   fprintf(outfile," Warning: L[18] = %6.2f, while Fed-line[38] = %6.2f\n", L[18], PrelimFedReturn.fedline[38] );
  } 

 L[19] = L[18];
 showline(19);

 GetLineF( "L20", &L[20] );	/* Interest income from non-NY state or local bonds */

 GetLineF( "L21", &L[21] );	/* Public employee retirement contributions (pg 16) */

 GetLineF( "L22", &L[22] );	/* College choice tuition saving distributions */

 GetLineF( "L23", &L[23] );	/* Other (pg 17) */

 for (j=19; j<=23; j++) L[24] = L[24] + L[j];
 showline(24);

 L[25] = L[4];			/* Taxable refunds, credits, offsets */
 showline(25);

 GetLineF( "L26", &L[26] );	/* Pensions of NYS and local governments and the federal government (see page 24)  */

 L[27] = L[14];			/* Taxable amount of social security benefits */
 showline(27);

 GetLineF( "L28", &L[28] );	/* Interest income on U.S. government bonds */

 GetLine( "L29", &L[29] );	/* Pension and annuity income exclusion  */
 if (L[29] > 20000.0)
  {
   L[29] = 20000.0;
   showline_wmsg(29,"(Limited to 20,000.)");
  }
 else
  showline(29);

 GetLineF( "L30", &L[30] );	/* College choice tuition savings deduction / earnings distributions */

 GetLineF( "L31", &L[31] );	/* Other (see page 21) */

 for (j=25; j<=31; j++) L[32] = L[32] + L[j];
 showline(32);

 L[33] = L[24] - L[32];
 showline_wmsg(33,"New York adjusted gross income (AGI)");

 /* NYS Itemized Deductions Worksheet */
 GetLine( "LTcare%", &LTC );
 GetLine( "AddAdj", &AddAdj );
 GetLine( "CollegeDed", &CollegeDed );
 dedws_a = PrelimFedReturn.schedA[4];	/* (a) */
 dedws_b = PrelimFedReturn.schedA[9];	/* (b) */
 dedws_c = PrelimFedReturn.schedA[15];	/* (c) */
 dedws_d = PrelimFedReturn.schedA[19];	/* (d) */
 dedws_e = PrelimFedReturn.schedA[20];	/* (e) */
 dedws_f = PrelimFedReturn.schedA[27];	/* (f) */
 dedws_g = PrelimFedReturn.schedA[28];	/* (g) */
 dedws_h = PrelimFedReturn.schedA[29];	/* (h) */
 dedws_i = PrelimFedReturn.schedA[5] + PrelimFedReturn.schedA[8] 
		+ LTC * PrelimFedReturn.schedA[4];	/* (i) */
 dedws_j = dedws_h - dedws_i; 	/* (j) */
 dedws_k = AddAdj;		  	/* (k) */
 dedws_l = dedws_j + dedws_k;	/* (l) */
 if (L[33] <= 100000.0)
  dedws_m = 0.0;			/* (m) */
 else
  { /*L33_morethan_100000*/
   double ws[50];
   if (L[33] <= 475000.0)
    { /* Worksheet 3 pg 27 */
      ws[1] = L[33];
      switch (status)
       {
        case SINGLE:  case MARRIED_FILLING_SEPARAT: ws[2] = 100000.0; break;
        case HEAD_OF_HOUSEHOLD:                     ws[2] = 150000.0; break;
        case MARRIED_FILLING_JOINTLY:  case WIDOW:  ws[2] = 200000.0; break;
	default: ws[2] = 0.0;
       }
      ws[3] = ws[1] - ws[2];
      if (ws[3] < 0.0)
	dedws_m = 0.0;
      else
       {
	ws[4] = smallerof( ws[3], 50000.0 );
	ws[5] = 0.0001 * Round( 10000.0 * (ws[4] / 50000) );
	ws[6] = 0.25 * dedws_a;
	ws[7] = ws[5] * ws[6];
	dedws_m = ws[7];
       }
    }
   else
   if (L[33] <= 525000.0)
    { /* Worksheet 4 pg 27 */
      ws[1] = L[33] - 475000.0;
      ws[2] = 0.0001 * Round( 10000.0 * (ws[1] / 50000) );
      ws[3] = 0.25 * dedws_a;
      ws[4] = ws[2] * ws[3];
      ws[5] = ws[3] + ws[4];
      dedws_m = ws[5];  
    }
   else
   if (L[33] <= 1000000.0)
    dedws_m = 0.5 * dedws_l;
   else
   if (L[33] <= 10000000.0)
    { /* Worksheet 5 pg 27 */
      ws[1] = L[33];
      ws[2] = 0.5 * dedws_d;
      ws[3] = ws[1] - ws[2];
      dedws_m = ws[3];  
    }
   else
    { /* Worksheet 6 pg 27 */
      ws[1] = L[33];
      ws[2] = 0.25 * dedws_d;
      ws[3] = ws[1] - ws[2];
      dedws_m = ws[3];  
    }
   } /*L33_morethan_100000*/
 dedws_n = dedws_l - dedws_m;	 /* (n) */
 dedws_o = CollegeDed;
 dedws_p = dedws_n + dedws_o;
 itemized_ded = dedws_p;
 // GetLine( "Itemized_Deduction", &itemized_ded );     /* Form IT-201-ATT or 0.0 if std ded. */

 switch (status)	/* Determine the Std. Deduction. */
  {
   case SINGLE: if (Dependent)   std_ded = 3000.0; else std_ded = 7500.0; break;
   case MARRIED_FILLING_JOINTLY: std_ded = 15000.0; break;
   case MARRIED_FILLING_SEPARAT: std_ded =  7500.0; break;
   case HEAD_OF_HOUSEHOLD: 	 std_ded = 10500.0; break;
   case WIDOW: 			 std_ded = 15000.0; break;
  }

 if (std_ded > itemized_ded) 
  {
   L[34] = std_ded; 
   showline_wmsg(34,"(Mark std-ded)");
  }
 else
  {
   L[34] = itemized_ded;
   showline_wmsg(34,"(Mark itemized-ded)");
  }

 L[35] = L[33] - L[34];
 if (L[35] < 0.0) L[35] = 0.0;
 else showline(35);

 get_parameter( infile, 's', word, "L36" );	/* Number of Dependent Exemptions (Pg 76, line e) */
 get_parameters( infile, 'i', &k, "L36" );
 L[36] = 1000.0 * (double)k;
 showline(36);
 
 L[37] = L[35] - L[36];
 if (L[37] < 0.0)
  {
   L[37] = 0.0;
   L[38] = L[37];
  }
 else 
  {
   showline_wmsg(37,"taxable income");
   L[38] = L[37];
   showline(38);
  }

 if (L[33] <= 100000.0)
   L[39] = TaxRateLookup( L[38], status );
 else
   L[39] = tax_computation_worksheet( status );
 showline(39);

 /* Household credit. */
 get_parameter( infile, 's', word, "Exemptions" );	/* NY dependent exemptions, Pg 29. */
 get_parameter( infile, 'i', &Exemptions, "Exemptions" );
 if (Dependent)
  L[40] = 0.0;
 else	/* From tables on page 29. */
 if (status==SINGLE)
  {
   if (L[18] <  5000.0) L[40] = 75.0; else
   if (L[18] <  6000.0) L[40] = 60.0; else
   if (L[18] <  7000.0) L[40] = 50.0; else
   if (L[18] < 20000.0) L[40] = 45.0; else
   if (L[18] < 25000.0) L[40] = 40.0; else
   if (L[18] < 28000.0) L[40] = 20.0; else  L[40] = 0.0;
  }
 else	/* Status = MARRIED_FILLING_JOINTLY, MARRIED_FILLING_SEPARAT, Head_of_house, Widow */
  if (status!=MARRIED_FILLING_SEPARAT)
   {
    if (L[18] <  5000.0) L[40] = 90.0 + 15.0 * (Exemptions-1); else
    if (L[18] <  6000.0) L[40] = 75.0 + 15.0 * (Exemptions-1); else
    if (L[18] <  7000.0) L[40] = 65.0 + 15.0 * (Exemptions-1); else
    if (L[18] < 20000.0) L[40] = 60.0 + 15.0 * (Exemptions-1); else
    if (L[18] < 22000.0) L[40] = 60.0 + 10.0 * (Exemptions-1); else
    if (L[18] < 25000.0) L[40] = 50.0 + 10.0 * (Exemptions-1); else
    if (L[18] < 28000.0) L[40] = 40.0 +  5.0 * (Exemptions-1); else  
    if (L[18] < 32000.0) L[40] = 20.0 +  5.0 * (Exemptions-1); else  L[40] = 0.0;
   }
  else
   {
    if (L[18] <  5000.0) L[40] = 45.0 + 7.5 * (Exemptions-1); else
    if (L[18] <  6000.0) L[40] = 37.5 + 7.5 * (Exemptions-1); else
    if (L[18] <  7000.0) L[40] = 32.5 + 7.5 * (Exemptions-1); else
    if (L[18] < 20000.0) L[40] = 30.0 + 7.5 * (Exemptions-1); else
    if (L[18] < 22000.0) L[40] = 30.0 + 5.0 * (Exemptions-1); else
    if (L[18] < 25000.0) L[40] = 25.0 + 5.0 * (Exemptions-1); else
    if (L[18] < 28000.0) L[40] = 20.0 + 2.5 * (Exemptions-1); else  
    if (L[18] < 32000.0) L[40] = 10.0 + 2.5 * (Exemptions-1); else  L[40] = 0.0;
   }
 showline_wmsg(40,"NY state household credit");	/* NY state household credit, (pg 29). */

 GetLineF( "L41", &L[41] );	/* Resident credit, Form IT-112-R or IT-112-C, pg 98 */

 GetLineF( "L42", &L[42] );	/* Other New York State nonrefundable credits  */

 L[43] = L[40] + L[41] + L[42];
 showline(43);

 L[44] = L[39] - L[43];
 if (L[44] < 0.0) L[44] = 0.0;
 else showline(44);

 GetLineF( "L45", &L[45] );	/* Net Other New York State taxes, Form IT-201-ATT, line 30 */

 L[46] = L[44] + L[45];
 showline_wmsg(46,"Total New York State taxes");

 get_parameter( infile, 's', word, "NYC_Resident" );
 get_parameters( infile, 'b', &nyc_resident, "NYC_Resident (yes/no) ?"); 

 GetLine( "L50", &L[50] );	/* Part-year New York City resident tax */
 GetLine( "L51", &L[51] );	/* Other New York City taxes */
 GetLine( "L53", &L[53] );	/* NY City nonrefundable credits */

 if (nyc_resident)
  { /*NYC*/
   L[47] = NYcityTaxRateFunction( L[38], status ); 
   showline(47);

   /* NYC Household credit. */
   if (Dependent) L[48] = 0.0;
   else
   if (status==SINGLE)		/* From table 4, 5, or 6 on page 30. */
    {
     if (L[18] <  10000.0) L[48] = 15.0; else
     if (L[18] <  12500.0) L[48] = 10.0; else  L[48] = 0.0;
    }
   else	/* Status = 2, 4, or 5. */
   if (status!=MARRIED_FILLING_SEPARAT)
    {
     if (L[18] <  15000.0) L[48] = 30.0 * Exemptions; else
     if (L[18] <  17500.0) L[48] = 25.0 * Exemptions; else
     if (L[18] <  20000.0) L[48] = 15.0 * Exemptions; else
     if (L[18] <  22500.0) L[48] = 10.0 * Exemptions; else  L[48] = 0.0;
    }
   else
    {
     if (L[18] <  15000.0) L[48] = 15.0 * Exemptions; else
     if (L[18] <  17500.0) L[48] = 12.5 * Exemptions; else
     if (L[18] <  20000.0) L[48] =  7.5 * Exemptions; else
     if (L[18] <  22500.0) L[48] =  5.0 * Exemptions; else  L[48] = 0.0;
    }
   showline_wmsg(48,"NY City household credit");	/* NY City household credit, (pg 34). */

   L[49] = L[47] - L[48];
   if (L[49] > 0.0) showline(49);
   else L[49] = 0.0;

   showline(50);
   showline(51);

   L[52] = L[49] + L[50] + L[51];
   showline(52);
   showline(53);

   L[54] = L[52] - L[53];
   if (L[54] > 0.0) showline(54);  else  L[54] = 0.0;

   /* Yonkers case not added. */

   L[58] = L[54] + L[55] + L[56] + L[57];
   showline_wmsg(58,"NYC taxes");
  } /*NYC*/

 GetLineF( "L59", &L[59] );	/* Sales or use tax, see pg 33. */

 GetLineF( "L60", &L[60] );	/* Voluntary Gift contibutions (pgs 34). */

 L[61] = L[46] + L[58] + L[59] + L[60];
 showline(61);

 L[62] = L[61];
 showline(62);

 GetLineF( "L63", &L[63] );	/* Empire State child credit (attach new Form IT-213) */

 GetLineF( "L64", &L[64] );	/* NYS/NYC Child care/dependent credit. Form IT-216. */

 GetLineF( "L65", &L[65] );	/* NYS Earned income credit. Form IT-213. */

 GetLineF( "L66", &L[66] );	/* NYS State noncustodial parent EIC (attach new Form IT-209) */

 GetLineF( "L67", &L[67] );	/* Real property credit. Form IT-214. */

 GetLineF( "L68", &L[68] );	/* College tution credit. Form IT-272. */

 if (nyc_resident)		/* City of NY school taxcredit. (pg 101) */
  { /*NYC*/
    if (Dependent) L[69] = 0.0;
    else
    if (L[37] < 250000)
     {
      if ((status==SINGLE) || (status==MARRIED_FILLING_SEPARAT) || (status==HEAD_OF_HOUSEHOLD))  L[69] = 62.5;  else
      if ((status==MARRIED_FILLING_JOINTLY) || (status==WIDOW))  L[69] = 125.0;
     }
    else
     L[69] = 0.0;
    showline(69);
    /* L[70] = earned_income_credit; */
    /* showline(70); */
  } /*NYC*/

 GetLineF( "L71", &L[71] );	/* Other refundable credits, IT-201-ATT line 18) */

 GetLineF( "L72", &L[72] );	/* Total NY State tax withheld. */

 GetLineF( "L73", &L[73] );	/* Total City of NY tax withheld. */
 GetLineF( "L74", &L[74] );	/* Yonkers tax withheld. */
 
 GetLineF( "L75", &L[75] );	/* Total estimated tax payments (from IT-370)*/

 for (j=63; j<=75; j++) L[76] = L[76] + L[j];
 showline(76);

 if (L[76] > L[62])
  {
   L[77] = L[76] - L[62];
   fprintf(outfile,"L77 = %6.2f	REFUND !!!\n", L[77] );
   L[78] = L[77];
   showline(78);
  }
 else
  {
   L[80] = L[62] - L[76];
   fprintf(outfile,"L80 = %6.2f	YOU OWE !!!\n", L[80] );
  }
 
 fclose(infile);
 fclose(outfile);
 printf("\nListing results from file: %s\n\n", outfname);
 Display_File( outfname );
 return 0;
}
