/************************************************************************/
/* TaxSolve_OH_IT1040_2011.c - 						*/
/* Copyright (C) 2011 - Aston Roberts					*/
/* 									*/
/* Compile:   gcc taxsolve_OH_IT1040_2011.c -o taxsolve_OH_IT1040_2011	*/
/* Run:	      ./taxsolve_OH_IT1040_2011  OH_IT1040_2011.txt 		*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-11-2012	aston_roberts@yahoo.com			*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"

double thisversion=10.0;

#define SINGLE 		        1
#define MARRIED_FILLING_JOINTLY 2
#define MARRIED_FILLING_SEPARAT 3
#define HEAD_OF_HOUSEHOLD       1

double TaxRateFunction( double x, int status )
{
 if (x <  5200.0) return x * 0.00587; else
 if (x < 10400.0) return  30.52 + (x-5200.0) * 0.01174; else
 if (x < 15650.0) return  91.57 + (x-10400.0) * 0.02348; else
 if (x < 20900.0) return 214.84 + (x-15650.0) * 0.02935; else
 if (x < 41700.0) return 368.93 + (x-20900.0) * 0.03521; else
 if (x < 83350.0) return 1101.30 + (x-41700.0) * 0.04109; else
 if (x < 104250.0) return 2812.70 + (x-83350.0) * 0.04695; else
 if (x < 208500.0) return 3793.96 + (x-104250.0) * 0.05451; else
 return 9476.63 + (x-208500.0) * 0.05925; 
}



/*----------------------------------------------------------------------------*/

int main( int argc, char *argv[] )
{
 int i, j, k;
 char word[1000], outfname[1000];
 int status=0, exemptions=0, qualify_jfc=0;
 time_t now;
 double factor60, factor65, limit;
 double L21a=0.0, L21b=0.0, L21c=0.0, L21d=0.0;
 double L33a=0.0, L33b=0.0, L33c=0.0, L33d=0.0, L33e=0.0, L33f=0.0, L33g=0.0;
 double L35a=0.0, L35b=0.0, L37a=0.0, L37b=0.0;
 double L38a=0.0, L38b=0.0, L38c=0.0, L43a=0.0, L43b=0.0, L43c=0.0;
 double jfc;

 /* Intercept any command-line arguments. */
 printf("OH IT1040 2011 - v%3.1f\n", thisversion);
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  verbose = 1;
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[i]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,"_out.txt"); else strcpy(&(outfname[j]),"_out.txt");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (i=0; i<MAX_LINES; i++) 
  {
   L[i] = 0.0;
  }

 /* Accept parameters from input file. */
 /* Expect  OH IT1040 lines, something like:
	Title:  OH IT1040 1999 Return
	L1		{Wages}
*/

 /* Accept Form's "Title" line, and put out with date-stamp for records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s		%s\n", word, ctime( &now ));

 /* get_parameter(infile, kind, x, emssg ) */
 get_parameter( infile, 's', word, "Status" );
 get_parameter( infile, 'l', word, "Status ?");
 if (strncasecmp(word,"Single",4)==0) status = SINGLE; else
 if (strncasecmp(word,"Married/Joint",11)==0) status = MARRIED_FILLING_JOINTLY; else
 if (strncasecmp(word,"Married/Sep",11)==0) status = MARRIED_FILLING_SEPARAT; else
 if (strncasecmp(word,"Head_of_House",4)==0) status = HEAD_OF_HOUSEHOLD;
 else
  { 
   printf("Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house.\nExiting.\n", word); 
   fprintf(outfile,"Error: unrecognized status '%s'. Must be: Single, Married/joint, Married/sep, Head_of_house.\nExiting.\n", word); 
   exit(1); 
  }
 fprintf(outfile,"Status = %s (%d)\n", word, status);

 GetLine( "L1", &L[1] );	/* Federal Adjusted Gross Income */

 get_parameter( infile, 's', word, "Exemptions" );	/* Exemptions, self/depend. */
 get_parameters( infile, 'i', &exemptions, "Exemptions"); 

 /* Answer YES only if Married Filing Jointly, and you and your spouse */
 /* each have qualifying Ohio adjusted gross income of at least $500. */
 get_parameter( infile, 's', word, "JointCredit" );
 get_parameter( infile, 'b', &qualify_jfc, "JointCredit ?"); 
 
 // GetLine( "L13", &L[13] );	/* Resident/Nonres/Part-Yr Credits (Sch. C/D) & Nonrefund. Bus. Credits (Sch. E) */
 GetLine( "L14", &L[14] );	/* Manufacturing equipment grant. */
 GetLine( "L17", &L[17] );	/* Unpaid Ohio Use Tax */
 GetLine( "L19", &L[19] );	/* Ohio Tax Withheld (box 17 on your W-2) */
 GetLine( "L20", &L[20] );	/* Ohio Estimated Tax, IT 40P Payments for 2011, ... */
 GetLine( "L21a", &L21a );	/* Refundable Business Jobs Credits, */
 GetLine( "L21b", &L21b );	/* Refundable Pass-through Entity Credits, */
 GetLine( "L21c", &L21c );	/* Refundable Histroic Preservation Credits, */
 GetLine( "L21d", &L21d );	/* Refundable Motion Picture Credits */
 L[21] = L21a + L21b + L21c + L21d;

 /* Additions - add to extent not included in fed adjusted gross income (line 1). */
 GetLine( "L31", &L[31] );	/* Non-Ohio state or local gov't interest, dividends. */
 GetLine( "L32", &L[32] );	/* Pass-through Entity addback. */

 GetLine( "L33a", &L33a );	/* Fed int+div subject to state tax & misc. fed adj. */
 GetLine( "L33b", &L33b );	/* Reimbursed college tuit. fees deducted prev yrs. */
 GetLine( "L33c", &L33c );	/* Losses, sale of Ohio Public Obligations. */
 GetLine( "L33d", &L33d );	/* Nonmedical withdrawals OH Med. Savings Acct. */
 GetLine( "L33e", &L33e );	/* Reimb. exp. prev. deducted, if reimb. not in FAGI .*/
 GetLine( "L33f", &L33f );	/* Lump sum distribution add-back + misc. fed income tax adjustments */
 GetLine( "L33g", &L33g );	/* 5/6 adjustment for IRC section 168(k) and 179 depreciation expense */
 L[33] = L33a + L33b + L33c + L33d + L33e + L33f + L33g;

 /* Deductions - see limitations in instructions. */
 GetLine( "L35a", &L35a );	/* Fed int + div exempt from state taxation. */
 GetLine( "L35b", &L35b );	/* Adjustment for Internal Revenue Code sections 168(k) + 179 deprec. expense */
 L[35] = L35a + L35b;
 GetLine( "L36", &L[36] );	/* Employee compensation earned in OH by non-residents. */
 GetLine( "L37a", &L37a );	/* Military pay. */
 GetLine( "L37b", &L37b );	/* Military Retirement Income. */
 L[37] = L37a + L37b;
 GetLine( "L38a", &L38a );	/* State/municipal income tax overpayments. */
 GetLine( "L38b", &L38b );	/* Refund or reimbursements shown on IRS form 1040, line 21 */
 GetLine( "L38c", &L38c );	/* Repayment of income reported in a prior year, ... */
 L[38] = L38a + L38b + L38c;
 GetLine( "L39", &L[39] );	/* Disab. & survivor benef. */
 GetLine( "L40", &L[40] );	/* Qualifying soc. sec. + railroad benefits. */ 
 GetLine( "L41", &L[41] );	/* Education: Ohio 529 contributions */ 
 GetLine( "L42", &L[42] );	/* Ohio National Guard reimbursements */
 GetLine( "L43a", &L43a );	/* Unreimbursed health insurance ... */
 GetLine( "L43b", &L43b );	/* Funds deposited into, and earnings of, a medical savings account */
 GetLine( "L43c", &L43c );	/* Qualified organ donor expenses */
 L[43] = L43a + L43b + L43c;
 GetLine( "L44", &L[44] );	/* Wage expense not deducted */
 
 /* Other. Check all that apply: */
 GetLine( "L45", &L[45] );	/* Interest income from Ohio Public Obligations ... */

 GetLine( "L48", &L[48] );	/* Retirement Income Credit (see instructions for credit table) (Limit $200) */
 if (L[48] > 200.0) L[48] = 200.0;
 GetLine( "L49", &L[49] );	/* Senior Citizen Credit (Limit $50 per return) */
 if (L[49] > 50.0) L[49] = 50.0;
 GetLine( "L50", &L[50] );	/* Lump Sum Distribution Credit (you must be 65 years of age or older to claim this credit) */
 GetLine( "L51", &L[51] );	/* Child and Dependent Care Credit (see instructions and worksheet) */
 GetLine( "L52", &L[52] );	/* Lump Sum Retirement Credit */
 GetLine( "L54", &L[54] );	/* Displaced worker credit (Limit $500 single; $1,000 joint, if both spouses qualify) */
 if (status == MARRIED_FILLING_JOINTLY)
  limit = 2.0 * 500.0;
 else
  limit = 500.0;
 L[54] = smallerof( L[54], limit );

 GetLine( "L55", &L[55] );	/* Ohio Political Contributions Credit (Limit $50 single; $100 joint) */
 if (status == MARRIED_FILLING_JOINTLY)
  limit = 2.0 * 50.0;
 else
  limit = 50.0;
 L[55] = smallerof( L[55], limit );

 GetLine( "L56", &L[56] );	/* Ohio Adoption Credit (Limit $1500 per adoption) */

 GetLine( "L58", &L[58] );	/* Portion of line 3 subjected to tax by other states. */
 GetLine( "L61", &L[61] );	/* 2011 income tax paid to other states ... */

 /* For non-residents only. */
 GetLine( "L63", &L[63] );	/* Portion Ohio Adj. Gross Income not earned in Oh. */

 GetLine( "L66", &L[66] );	/* Nonrefundable Business Credits, Sched-E line 10 */




 /* ---- Do Calculations. ---- */

 L[34] = L[31] + L[32] + L[33];

 for (j=35; j<=45; j++)
  L[46] = L[46] + L[j];

 L[47] = L[34] - L[46];
 L[2] = L[47];

 L[3] = L[1] + L[2];
 L[4] = 1650.0 * exemptions;
 L[5] = NotLessThanZero( L[3] - L[4] );
 L[6] = TaxRateFunction( L[5], status );

 if (L[5] <= 10000.0) L[53] = 88.0;

 for (j=48; j<=56; j++)
  L[57] = L[57] + L[j];
 L[7] = L[57];

 L[8] = NotLessThanZero( L[6] - L[7] );
 L[9] = 20.0 * exemptions;
 L[10] = NotLessThanZero( L[8] - L[9] );

 if ((status == MARRIED_FILLING_JOINTLY) && (qualify_jfc))
  { /*Joint_Filing_Credit*/
    if (L[5] < 25000) jfc = 0.20;
    else
    if (L[5] < 50000) jfc = 0.15;
    else
    if (L[5] < 75000) jfc = 0.10;
    else jfc = 0.05;
    L[11] = smallerof( jfc * L[10], 650.0 );
  } /*Joint_Filing_Credit*/

 L[12] = L[10] - L[11];

 /* L[15] computed below, which depends on L[69] via L13]. */

 L[59] = L[3];
 factor60 = 0.0001 * (int)(10000.0 * (L[58] / L[59]));	/* 4-digits right of decimal-pt. */
 L[60] = factor60 * L[12];
 L[62] = smallerof( L[60], L[61] );
 L[67] = L[62];

 L[64] = L[3];
 factor65 = 0.0001 * (int)(10000.0 * (L[63] / L[64]));  /* 4-digits right of decimal-pt. */
 L[65] = factor65 * L[12];
 L[68] = L[65];

 L[67] = L[62];
 L[69] = L[66] + L[67] + L[68];
 L[13] = L[69];

 L[15] = NotLessThanZero( L[12] - (L[13] + L[14]) );
 L[18] = L[15] + L[16] + L[17];
 L[22] = L[19] + L[20] + L[21];

 if ((status == SINGLE) && (L[1] <= 11650.0) && (L[47] == 0.0))
  fprintf(outfile, "You do not need to file Ohio tax return (Fed AGI < minimum).\n");

 if ((status == MARRIED_FILLING_JOINTLY) && (L[1] <= 13300.0) && (L[47] == 0.0))
  fprintf(outfile, "You do not need to file Ohio tax return (Fed AGI < minimum).\n");

 if (L[4] >= L[3])
  fprintf(outfile, "You do not need to file Ohio tax return (L[4] >= L[3]).\n");


 /* Output the Results. */

 showline(1);
 showline(2);
 showline(3);
 fprintf(outfile,"Exemptions = %d\n", exemptions );
 for (j=4; j<=10; j++)
  showline(j);
 fprintf(outfile, "L10a = %6.2f\n", L[10]);
 for (j=11; j<=17; j++)
  showline(j);
 showline_wmsg(18, "Total Ohio Tax");
 showline(19);
 showline(20);
 showline_wlabel( "L21a", L21a );
 showline_wlabel( "L21b", L21b );
 showline_wlabel( "L21c", L21c );
 showline_wlabel( "L21d", L21d );
 showline_wmsg(22, "Total Payments");
 if (L[22] > L[18])
  {
   L[23] = L[22] - L[18];
   showline_wmsg(23, "AMOUNT OVERPAID");
  }
 else
  {
   L[27] = L[18] - L[22];
   showline_wmsg(27, "AMOUNT DUE");
  }
 showline(31);
 showline(32);
 showline_wlabel( "L33a", L33a );
 showline_wlabel( "L33b", L33b );
 showline_wlabel( "L33c", L33c );
 showline_wlabel( "L33d", L33d );
 showline_wlabel( "L33e", L33e );
 showline_wlabel( "L33f", L33f );
 showline_wlabel( "L33g", L33g );
 showline_wmsg(34,"Total additions");
 showline_wlabel( "L35a", L35a );
 showline_wlabel( "L35b", L35b );
 showline(36);
 showline_wlabel( "L37a", L37a );
 showline_wlabel( "L37b", L37b );
 showline_wlabel( "L38a", L38a );
 showline_wlabel( "L38b", L38b );
 showline_wlabel( "L38c", L38c );
 showline(39);
 showline(40);
 showline(41);
 showline(42);
 showline_wlabel( "L43a", L43a );
 showline_wlabel( "L43b", L43b );
 showline_wlabel( "L43c", L43c );
 showline(44);
 showline(45);
 showline_wmsg(46,"Total Deductions");
 for (j=47; j<=56; j++)
  showline(j);
 showline_wmsg(57,"Total Credits");
 for (j=58; j<=59; j++)
  showline(j);
 fprintf(outfile," (L60 [%g]) \t", factor60 );
 for (j=60; j<=64; j++)
  showline(j);
 fprintf(outfile," (L65 [%g]) \t", factor65 );
 for (j=65; j<=69; j++)
  showline(j);

 fclose(infile);
 fclose(outfile);
 Display_File( outfname );
 printf("\nResults written to file:  %s\n", outfname);
 return 0;
}
