/************************************************************************/
/* TaxSolve_US_8829.c -		 					*/
/* Copyright (C)  2013 - Rev Tactule					*/
/* 									*/
/* Compile:								*/
/*  cc taxsolve_US_8829.c -o taxsolve_US_8829				*/
/*									*/
/* Documentation & Updates:						*/
/*        http://opentaxsolver.sourceforge.net/				*/
/*									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Rev Tactule revenatic@yahoo.com - April 14, 2009			*/
/************************************************************************/

float thisversion=10.0;

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "taxsolve_routines.c"

enum{
	L1 = 1,
	L2,
	L3,
	L4,
	L5,
	L6,
	L7,
	L8,

	L9a,
	L10a,
	L11a,
	L12a,

	L9b,
	L10b,
	L11b,
	L12b,

	L13,
	L14,
	L15,

	L16a,
	L17a,
	L18a,
	L19a,
	L20a,
	L21a,
	L22a,

	L16b,
	L17b,
	L18b,
	L19b,
	L20b,
	L21b,
	L22b,

	L23,
	L24,
	L25,
	L26,
	L27,

	L28,
	L29,
	L30,

	L31,
	L32,
	L33,
	L34,
	L35,

	L36,
	L37,
	L38,
	L39,

	L40,
	L41,

	L42,
	L43,

	LINE_COUNT
};

void showline_a(int p){
	if(p >= 9 && p <= 12){
		fprintf(outfile,"L%da = %6.2f\n", p, L[L9a + p - 9]);
	}
	else if(p >= 16 && p <= 22){
		fprintf(outfile,"L%da = %6.2f\n", p, L[L16a + p - 16]);
	}
}

void showline_b(int p){
	if(p >= 9 && p <= 12){
		fprintf(outfile,"L%db = %6.2f\n", p, L[L9b + p - 9]);
	}
	else if(p >= 16 && p  <= 22){
		fprintf(outfile,"L%db = %6.2f\n", p, L[L16b + p - 16]);
	}
}

void m_showline(int p){
	int index = p;
	if(p >= L23){
		index = p - L23 + 23;
	}
	else if(p >= L13 && p <= L15){
		index = p - L13 + 13;
	}
	fprintf(outfile,"L%d = %6.2f\n", index, L[p]);
}

/*----------------------------------------------------------------------------*/

int main ( int argc, char *argv[] )
{
 int i, j, k;
 char word[2000], outfname[1000];
 time_t now;

 printf("US 8829 2012 - v%3.2f\n", thisversion);

 /* Decode any command-line arguments. */
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  { verbose = 1; }
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfname,argv[i]);
    j = strlen(outfname)-1;
    while ((j>=0) && (outfname[j]!='.')) j--;
    if (j<0) strcat(outfname,"_out.txt"); else strcpy(&(outfname[j]),"_out.txt");
    outfile = fopen(outfname,"w");
    if (outfile==0) {printf("ERROR: Output file '%s' could not be opened.\n", outfname); exit(1);}
    printf("Writing results to file:  %s\n", outfname);
   }
  else
   {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}

 /* Pre-initialize all lines to zeros. */
 for (i=0; i<LINE_COUNT; i++) { L[i] = 0.0; }

 /* Accept parameters from input file. */
 /* Expect  Sched C lines, something like:
	Title:  Sched C 1999 Return
	L1		{Gross Receipts}
	L2		{Returns and Allowances}
	. . .
 */

 /* Accept Form's "Title" line, and put out with date-stamp for your records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outfile,"\n%s 	%s\n", word, ctime( &now ));

 GetLine( "L1", &L[L1] );	/* Square Area used for Business */

 GetLine( "L2", &L[L2] );	/* Total Area of Home */	


 GetLine( "L4", &L[L4] );	/* Daycare hours per day * total days */	

 GetLine( "L5", &L[L5] );	/* Total Hours in Year. */	

 GetLine( "L8", &L[L8] );	/* Line 29 from Schedule C */


 /* Alternates as: */
 /* Direct Expenses */
 /* Indirect Expenses */
 GetLine( "L9a", &L[L9a] );	/* Casualty Losses */
 GetLine( "L9b", &L[L9b] );	/* Casualty Losses */

 GetLine( "L10a", &L[L10a] );	/* Deductible Mortgage Interest */
 GetLine( "L10b", &L[L10b] );	/* Deductible Mortgage Interest */

 GetLine( "L11a", &L[L11a] );	/* Real Estate Taxes */
 GetLine( "L11b", &L[L11b] );	/* Real Estate Taxes */

 GetLine( "L16a", &L[L16a] );	/* Excess Mortgage Interest */
 GetLine( "L16b", &L[L16b] );	/* Excess Mortgage Interest */
 GetLine( "L17a", &L[L17a] );	/* Insurance */
 GetLine( "L17b", &L[L17b] );	/* Insurance */
 GetLine( "L18a", &L[L18a] );	/* Rent */
 GetLine( "L18b", &L[L18b] );	/* Rent */

 GetLine( "L19a", &L[L19a] );	/* Repairs and Maintenance */
 GetLine( "L19b", &L[L19b] );	/* Repairs and Maintenance */
 GetLine( "L20a", &L[L20a] );	/* Utilities */
 GetLine( "L20b", &L[L20b] );	/* Utilities */
 GetLine( "L21a", &L[L21a] );	/* Other Expenses */
 GetLine( "L21b", &L[L21b] );	/* Other Expenses */

 GetLine( "L24", &L[L24] );	/* Carryover from 2007 8829, Line 42*/

 GetLine( "L28", &L[L28] );	/* Excess Casualty Losses. */
 GetLine( "L30", &L[L30] );	/* Carryover of excess casualty losses Line 43. */

 GetLine( "L34", &L[L34] );	/* Casualty loss portion from 27 or 31. */

 GetLine( "L36", &L[L36] );	/* Smaller of home's adjusted basis or FMV. */
 GetLine( "L37", &L[L37] );	/* Value of Land in line 36. */

 GetLine( "L40", &L[L40]);	/* Depreciation percentage. */

 fclose(infile);

 

 /* -- Compute the tax form -- */


 /* Square area calculation. */
 m_showline(L1);
 m_showline(L2);
 L[L3] = L[L1] / L[L2] * 100.0;
 m_showline(L3);
 L[L3] /= 100.0;
 L[L7] = L[L3];

 /* Daycare center usage calculation. */
 m_showline(L4);
 L[L6] = L[L4] / L[L5];
 m_showline(L6);
 if (L[L6] > 0)
   L[L7] *= L[L6];
 L[L7] *= 100.0;
 m_showline(L7);
 L[L7] /= 100.0;
 m_showline(L8);

 /* Part 3 Depreciation Calculation. (36-43) Show lines later. */
 L[L38] = L[L36] - L[L37];
 L[L39] = L[L38] * L[L7];
 L[L41] = L[L39] * L[L40] / 100.0;

 /* Expenses */
 for (i = 0; i < 3; ++i)
  {
	L[L12a] += L[L9a + i];
	L[L12b] += L[L9b + i];

	showline_a(9 + i);
	showline_b(9 + i);
  }

 showline_a(12);
 showline_b(12);

 L[L13] = L[L12b] * L[L7];
 m_showline(L13);
 L[L14] = L[L12a] + L[L13];
 m_showline(L14);

 L[L15] = L[L8] - L[L14];
 if(L[L15] < 0)
	L[L15] = 0;
 m_showline(L15);


 for (i = 0; i < 6; ++i)
  {
	L[L22a] += L[L16a + i];
	L[L22b] += L[L16b + i];
 	showline_a(16 + i);
 	showline_b(16 + i);
  }
 showline_a(22);
 showline_b(22);

 L[L23] = L[L22b] * L[L7];

 L[L25] = L[L22a] + L[L23] + L[L24];
 L[L26] = L[L25] < L[L15] ? L[L25] : L[L15];
 L[L27] = L[L15] - L[L26];

 L[L29] = L[L41];

 L[L31] = L[L28] + L[L29] + L[L30];
 L[L32] = L[L27] < L[L31] ? L[L27] : L[L31];

 L[L33] = L[L14] + L[L26] + L[L32];
 L[L35] = L[L33] - L[L34];

 L[L42] = L[L25] - L[L26];
 L[L43] = L[L32] - L[L31];

 for(i = L23; i <= L43; ++i)
  {
	 m_showline(i);
  }


 fclose(outfile);

 Display_File( outfname );
 printf("\nListed results from file: %s\n", outfname);

 return 0;
}
